@@ -1,7 +1,7 @@
 /*
   entry.hpp
 
-  $Id: entry.hpp,v 1.1.1.1 2001/05/15 00:25:00 evertonm Exp $
+  $Id: entry.hpp,v 1.2 2004/01/28 19:14:10 evertonm Exp $
  */
 
 #ifndef ENTRY_HPP
@@ -19,10 +19,14 @@ private:
   void spawn(const proto_map *map) const;
 
 public:
-  entry(proto_t pro, vector<proto_map*> *pro_l) 
+  entry(proto_t pro, vector<proto_map*> *pro_l, bool fragile) 
     {
       proto      = pro;
       proto_list = pro_l;
+
+      iterator<vector<proto_map*>,proto_map*> it(*proto_list);
+      for (it.start(); it.cont(); it.next())
+        it.get()->set_fragile(fragile);
     }
 
   void show() const;
