@@ -1,7 +1,7 @@
 /*
   util.h
 
-  $Id: util.h,v 1.4 2002/05/08 03:50:03 evertonm Exp $
+  $Id: util.h,v 1.5 2005/05/30 02:13:28 evertonm Exp $
  */
 
 #ifndef UTIL_H
@@ -27,15 +27,11 @@ char* safe_strdup(const char* str);
 void safe_strcpy(char *trg, const char *src, int trg_buf_size);
 
 int cd_root();
-void close_fds(int first_fd);
 int std_to_null();
 int daemonize();
 
 void socket_close(int fd); 
 
-void fdset(int fd, fd_set *fds, int *maxfd);
-void fdclear(int fd, fd_set *fds, int *maxfd);
-
 #define MAX(a, b) (((a) > (b)) ? (a) : (b))
 
 #endif /* UTIL_H */
